// Copyright 2023 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/contentwarehouse/v1/document_schema_service.proto

#include "google/cloud/contentwarehouse/v1/internal/document_schema_connection_impl.h"
#include "google/cloud/contentwarehouse/v1/internal/document_schema_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/pagination_range.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>

namespace google {
namespace cloud {
namespace contentwarehouse_v1_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

DocumentSchemaServiceConnectionImpl::DocumentSchemaServiceConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<contentwarehouse_v1_internal::DocumentSchemaServiceStub>
        stub,
    Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(
          std::move(options), DocumentSchemaServiceConnection::options())) {}

StatusOr<google::cloud::contentwarehouse::v1::DocumentSchema>
DocumentSchemaServiceConnectionImpl::CreateDocumentSchema(
    google::cloud::contentwarehouse::v1::CreateDocumentSchemaRequest const&
        request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->CreateDocumentSchema(request),
      [this](grpc::ClientContext& context,
             google::cloud::contentwarehouse::v1::
                 CreateDocumentSchemaRequest const& request) {
        return stub_->CreateDocumentSchema(context, request);
      },
      request, __func__);
}

StatusOr<google::cloud::contentwarehouse::v1::DocumentSchema>
DocumentSchemaServiceConnectionImpl::UpdateDocumentSchema(
    google::cloud::contentwarehouse::v1::UpdateDocumentSchemaRequest const&
        request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->UpdateDocumentSchema(request),
      [this](grpc::ClientContext& context,
             google::cloud::contentwarehouse::v1::
                 UpdateDocumentSchemaRequest const& request) {
        return stub_->UpdateDocumentSchema(context, request);
      },
      request, __func__);
}

StatusOr<google::cloud::contentwarehouse::v1::DocumentSchema>
DocumentSchemaServiceConnectionImpl::GetDocumentSchema(
    google::cloud::contentwarehouse::v1::GetDocumentSchemaRequest const&
        request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->GetDocumentSchema(request),
      [this](
          grpc::ClientContext& context,
          google::cloud::contentwarehouse::v1::GetDocumentSchemaRequest const&
              request) { return stub_->GetDocumentSchema(context, request); },
      request, __func__);
}

Status DocumentSchemaServiceConnectionImpl::DeleteDocumentSchema(
    google::cloud::contentwarehouse::v1::DeleteDocumentSchemaRequest const&
        request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->DeleteDocumentSchema(request),
      [this](grpc::ClientContext& context,
             google::cloud::contentwarehouse::v1::
                 DeleteDocumentSchemaRequest const& request) {
        return stub_->DeleteDocumentSchema(context, request);
      },
      request, __func__);
}

StreamRange<google::cloud::contentwarehouse::v1::DocumentSchema>
DocumentSchemaServiceConnectionImpl::ListDocumentSchemas(
    google::cloud::contentwarehouse::v1::ListDocumentSchemasRequest request) {
  request.clear_page_token();
  auto& stub = stub_;
  auto retry = std::shared_ptr<
      contentwarehouse_v1::DocumentSchemaServiceRetryPolicy const>(
      retry_policy());
  auto backoff = std::shared_ptr<BackoffPolicy const>(backoff_policy());
  auto idempotency = idempotency_policy()->ListDocumentSchemas(request);
  char const* function_name = __func__;
  return google::cloud::internal::MakePaginationRange<
      StreamRange<google::cloud::contentwarehouse::v1::DocumentSchema>>(
      std::move(request),
      [stub, retry, backoff, idempotency, function_name](
          google::cloud::contentwarehouse::v1::ListDocumentSchemasRequest const&
              r) {
        return google::cloud::internal::RetryLoop(
            retry->clone(), backoff->clone(), idempotency,
            [stub](grpc::ClientContext& context,
                   google::cloud::contentwarehouse::v1::
                       ListDocumentSchemasRequest const& request) {
              return stub->ListDocumentSchemas(context, request);
            },
            r, function_name);
      },
      [](google::cloud::contentwarehouse::v1::ListDocumentSchemasResponse r) {
        std::vector<google::cloud::contentwarehouse::v1::DocumentSchema> result(
            r.document_schemas().size());
        auto& messages = *r.mutable_document_schemas();
        std::move(messages.begin(), messages.end(), result.begin());
        return result;
      });
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace contentwarehouse_v1_internal
}  // namespace cloud
}  // namespace google
