// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/dialogflow/v2/version.proto

#include "google/cloud/dialogflow_es/internal/versions_connection_impl.h"
#include "google/cloud/dialogflow_es/internal/versions_option_defaults.h"
#include "google/cloud/background_threads.h"
#include "google/cloud/common_options.h"
#include "google/cloud/grpc_options.h"
#include "google/cloud/internal/pagination_range.h"
#include "google/cloud/internal/retry_loop.h"
#include <memory>

namespace google {
namespace cloud {
namespace dialogflow_es_internal {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

VersionsConnectionImpl::VersionsConnectionImpl(
    std::unique_ptr<google::cloud::BackgroundThreads> background,
    std::shared_ptr<dialogflow_es_internal::VersionsStub> stub, Options options)
    : background_(std::move(background)),
      stub_(std::move(stub)),
      options_(internal::MergeOptions(std::move(options),
                                      VersionsConnection::options())) {}

StreamRange<google::cloud::dialogflow::v2::Version>
VersionsConnectionImpl::ListVersions(
    google::cloud::dialogflow::v2::ListVersionsRequest request) {
  request.clear_page_token();
  auto& stub = stub_;
  auto retry =
      std::shared_ptr<dialogflow_es::VersionsRetryPolicy const>(retry_policy());
  auto backoff = std::shared_ptr<BackoffPolicy const>(backoff_policy());
  auto idempotency = idempotency_policy()->ListVersions(request);
  char const* function_name = __func__;
  return google::cloud::internal::MakePaginationRange<
      StreamRange<google::cloud::dialogflow::v2::Version>>(
      std::move(request),
      [stub, retry, backoff, idempotency, function_name](
          google::cloud::dialogflow::v2::ListVersionsRequest const& r) {
        return google::cloud::internal::RetryLoop(
            retry->clone(), backoff->clone(), idempotency,
            [stub](grpc::ClientContext& context,
                   google::cloud::dialogflow::v2::ListVersionsRequest const&
                       request) {
              return stub->ListVersions(context, request);
            },
            r, function_name);
      },
      [](google::cloud::dialogflow::v2::ListVersionsResponse r) {
        std::vector<google::cloud::dialogflow::v2::Version> result(
            r.versions().size());
        auto& messages = *r.mutable_versions();
        std::move(messages.begin(), messages.end(), result.begin());
        return result;
      });
}

StatusOr<google::cloud::dialogflow::v2::Version>
VersionsConnectionImpl::GetVersion(
    google::cloud::dialogflow::v2::GetVersionRequest const& request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->GetVersion(request),
      [this](grpc::ClientContext& context,
             google::cloud::dialogflow::v2::GetVersionRequest const& request) {
        return stub_->GetVersion(context, request);
      },
      request, __func__);
}

StatusOr<google::cloud::dialogflow::v2::Version>
VersionsConnectionImpl::CreateVersion(
    google::cloud::dialogflow::v2::CreateVersionRequest const& request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->CreateVersion(request),
      [this](
          grpc::ClientContext& context,
          google::cloud::dialogflow::v2::CreateVersionRequest const& request) {
        return stub_->CreateVersion(context, request);
      },
      request, __func__);
}

StatusOr<google::cloud::dialogflow::v2::Version>
VersionsConnectionImpl::UpdateVersion(
    google::cloud::dialogflow::v2::UpdateVersionRequest const& request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->UpdateVersion(request),
      [this](
          grpc::ClientContext& context,
          google::cloud::dialogflow::v2::UpdateVersionRequest const& request) {
        return stub_->UpdateVersion(context, request);
      },
      request, __func__);
}

Status VersionsConnectionImpl::DeleteVersion(
    google::cloud::dialogflow::v2::DeleteVersionRequest const& request) {
  return google::cloud::internal::RetryLoop(
      retry_policy(), backoff_policy(),
      idempotency_policy()->DeleteVersion(request),
      [this](
          grpc::ClientContext& context,
          google::cloud::dialogflow::v2::DeleteVersionRequest const& request) {
        return stub_->DeleteVersion(context, request);
      },
      request, __func__);
}

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace dialogflow_es_internal
}  // namespace cloud
}  // namespace google
