// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/cloud/iot/v1/device_manager.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IOT_V1_DEVICE_MANAGER_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IOT_V1_DEVICE_MANAGER_CONNECTION_H

#include "google/cloud/iot/v1/device_manager_connection_idempotency_policy.h"
#include "google/cloud/iot/v1/internal/device_manager_retry_traits.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/options.h"
#include "google/cloud/status_or.h"
#include "google/cloud/stream_range.h"
#include "google/cloud/version.h"
#include <google/cloud/iot/v1/device_manager.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace iot_v1 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

using DeviceManagerRetryPolicy =
    ::google::cloud::internal::TraitBasedRetryPolicy<
        iot_v1_internal::DeviceManagerRetryTraits>;

using DeviceManagerLimitedTimeRetryPolicy =
    ::google::cloud::internal::LimitedTimeRetryPolicy<
        iot_v1_internal::DeviceManagerRetryTraits>;

using DeviceManagerLimitedErrorCountRetryPolicy =
    ::google::cloud::internal::LimitedErrorCountRetryPolicy<
        iot_v1_internal::DeviceManagerRetryTraits>;

/**
 * The `DeviceManagerConnection` object for `DeviceManagerClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `DeviceManagerClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `DeviceManagerClient`.
 *
 * To create a concrete instance, see `MakeDeviceManagerConnection()`.
 *
 * For mocking, see `iot_v1_mocks::MockDeviceManagerConnection`.
 */
class DeviceManagerConnection {
 public:
  virtual ~DeviceManagerConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StatusOr<google::cloud::iot::v1::DeviceRegistry> CreateDeviceRegistry(
      google::cloud::iot::v1::CreateDeviceRegistryRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::DeviceRegistry> GetDeviceRegistry(
      google::cloud::iot::v1::GetDeviceRegistryRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::DeviceRegistry> UpdateDeviceRegistry(
      google::cloud::iot::v1::UpdateDeviceRegistryRequest const& request);

  virtual Status DeleteDeviceRegistry(
      google::cloud::iot::v1::DeleteDeviceRegistryRequest const& request);

  virtual StreamRange<google::cloud::iot::v1::DeviceRegistry>
  ListDeviceRegistries(
      google::cloud::iot::v1::ListDeviceRegistriesRequest request);

  virtual StatusOr<google::cloud::iot::v1::Device> CreateDevice(
      google::cloud::iot::v1::CreateDeviceRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::Device> GetDevice(
      google::cloud::iot::v1::GetDeviceRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::Device> UpdateDevice(
      google::cloud::iot::v1::UpdateDeviceRequest const& request);

  virtual Status DeleteDevice(
      google::cloud::iot::v1::DeleteDeviceRequest const& request);

  virtual StreamRange<google::cloud::iot::v1::Device> ListDevices(
      google::cloud::iot::v1::ListDevicesRequest request);

  virtual StatusOr<google::cloud::iot::v1::DeviceConfig>
  ModifyCloudToDeviceConfig(
      google::cloud::iot::v1::ModifyCloudToDeviceConfigRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::ListDeviceConfigVersionsResponse>
  ListDeviceConfigVersions(
      google::cloud::iot::v1::ListDeviceConfigVersionsRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::ListDeviceStatesResponse>
  ListDeviceStates(
      google::cloud::iot::v1::ListDeviceStatesRequest const& request);

  virtual StatusOr<google::iam::v1::Policy> SetIamPolicy(
      google::iam::v1::SetIamPolicyRequest const& request);

  virtual StatusOr<google::iam::v1::Policy> GetIamPolicy(
      google::iam::v1::GetIamPolicyRequest const& request);

  virtual StatusOr<google::iam::v1::TestIamPermissionsResponse>
  TestIamPermissions(google::iam::v1::TestIamPermissionsRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::SendCommandToDeviceResponse>
  SendCommandToDevice(
      google::cloud::iot::v1::SendCommandToDeviceRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::BindDeviceToGatewayResponse>
  BindDeviceToGateway(
      google::cloud::iot::v1::BindDeviceToGatewayRequest const& request);

  virtual StatusOr<google::cloud::iot::v1::UnbindDeviceFromGatewayResponse>
  UnbindDeviceFromGateway(
      google::cloud::iot::v1::UnbindDeviceFromGatewayRequest const& request);
};

/**
 * A factory function to construct an object of type `DeviceManagerConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of DeviceManagerClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `DeviceManagerConnection`. Expected options are any of the types in
 * the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::iot_v1::DeviceManagerPolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `DeviceManagerConnection` created by
 * this function.
 */
std::shared_ptr<DeviceManagerConnection> MakeDeviceManagerConnection(
    Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace iot_v1
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_IOT_V1_DEVICE_MANAGER_CONNECTION_H
