// Copyright 2022 Google LLC
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//      https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// Generated by the Codegen C++ plugin.
// If you make any local changes, they will be lost.
// source: google/devtools/cloudprofiler/v2/profiler.proto

#ifndef GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_PROFILER_V2_PROFILER_CONNECTION_H
#define GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_PROFILER_V2_PROFILER_CONNECTION_H

#include "google/cloud/profiler/v2/internal/profiler_retry_traits.h"
#include "google/cloud/profiler/v2/profiler_connection_idempotency_policy.h"
#include "google/cloud/backoff_policy.h"
#include "google/cloud/options.h"
#include "google/cloud/status_or.h"
#include "google/cloud/version.h"
#include <google/devtools/cloudprofiler/v2/profiler.pb.h>
#include <memory>

namespace google {
namespace cloud {
namespace profiler_v2 {
GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_BEGIN

using ProfilerServiceRetryPolicy =
    ::google::cloud::internal::TraitBasedRetryPolicy<
        profiler_v2_internal::ProfilerServiceRetryTraits>;

using ProfilerServiceLimitedTimeRetryPolicy =
    ::google::cloud::internal::LimitedTimeRetryPolicy<
        profiler_v2_internal::ProfilerServiceRetryTraits>;

using ProfilerServiceLimitedErrorCountRetryPolicy =
    ::google::cloud::internal::LimitedErrorCountRetryPolicy<
        profiler_v2_internal::ProfilerServiceRetryTraits>;

/**
 * The `ProfilerServiceConnection` object for `ProfilerServiceClient`.
 *
 * This interface defines virtual methods for each of the user-facing overload
 * sets in `ProfilerServiceClient`. This allows users to inject custom behavior
 * (e.g., with a Google Mock object) when writing tests that use objects of type
 * `ProfilerServiceClient`.
 *
 * To create a concrete instance, see `MakeProfilerServiceConnection()`.
 *
 * For mocking, see `profiler_v2_mocks::MockProfilerServiceConnection`.
 */
class ProfilerServiceConnection {
 public:
  virtual ~ProfilerServiceConnection() = 0;

  virtual Options options() { return Options{}; }

  virtual StatusOr<google::devtools::cloudprofiler::v2::Profile> CreateProfile(
      google::devtools::cloudprofiler::v2::CreateProfileRequest const& request);

  virtual StatusOr<google::devtools::cloudprofiler::v2::Profile>
  CreateOfflineProfile(
      google::devtools::cloudprofiler::v2::CreateOfflineProfileRequest const&
          request);

  virtual StatusOr<google::devtools::cloudprofiler::v2::Profile> UpdateProfile(
      google::devtools::cloudprofiler::v2::UpdateProfileRequest const& request);
};

/**
 * A factory function to construct an object of type
 * `ProfilerServiceConnection`.
 *
 * The returned connection object should not be used directly; instead it
 * should be passed as an argument to the constructor of ProfilerServiceClient.
 *
 * The optional @p options argument may be used to configure aspects of the
 * returned `ProfilerServiceConnection`. Expected options are any of the types
 * in the following option lists:
 *
 * - `google::cloud::CommonOptionList`
 * - `google::cloud::GrpcOptionList`
 * - `google::cloud::UnifiedCredentialsOptionList`
 * - `google::cloud::profiler_v2::ProfilerServicePolicyOptionList`
 *
 * @note Unexpected options will be ignored. To log unexpected options instead,
 *     set `GOOGLE_CLOUD_CPP_ENABLE_CLOG=yes` in the environment.
 *
 * @param options (optional) Configure the `ProfilerServiceConnection` created
 * by this function.
 */
std::shared_ptr<ProfilerServiceConnection> MakeProfilerServiceConnection(
    Options options = {});

GOOGLE_CLOUD_CPP_INLINE_NAMESPACE_END
}  // namespace profiler_v2
}  // namespace cloud
}  // namespace google

#endif  // GOOGLE_CLOUD_CPP_GOOGLE_CLOUD_PROFILER_V2_PROFILER_CONNECTION_H
